import { _ as _objectWithoutPropertiesLoose, a as _objectSpread2 } from '../_rollupPluginBabelHelpers-1f0bf8c2.js';
import { createComponent } from 'reakit-system/createComponent';
import { createHook } from 'reakit-system/createHook';
import 'reakit-utils/shallowEqual';
import { useRef, useCallback, useEffect } from 'react';
import { useForkRef } from 'reakit-utils/useForkRef';
import 'reakit-utils/isButton';
import { warning } from 'reakit-warning';
import { useLiveRef } from 'reakit-utils/useLiveRef';
import 'reakit-utils/isSelfTarget';
import 'reakit-utils/useIsomorphicEffect';
import 'reakit-utils/hasFocusWithin';
import 'reakit-utils/isPortalEvent';
import 'reakit-utils/getActiveElement';
import 'reakit-utils/canUseDOM';
import 'reakit-utils/tabbable';
import '../Role/Role.js';
import '../Tabbable/Tabbable.js';
import 'reakit-system/useCreateElement';
import 'reakit-utils/createOnKeyDown';
import 'reakit-utils/getDocument';
import 'reakit-utils/fireBlurEvent';
import 'reakit-utils/fireKeyboardEvent';
import 'reakit-utils/getNextActiveElementOnBlur';
import '../reverse-30eaa122.js';
import '../getCurrentId-5aa9849e.js';
import '../findEnabledItemById-8ddca752.js';
import '../__keys-6742f591.js';
import { useComposite } from '../Composite/Composite.js';
import { C as COMBOBOX_KEYS } from '../__keys-4a2bf4ed.js';
import { g as getMenuId } from '../getMenuId-34730bd3.js';

function getControls(baseId, ariaControls) {
  var menuId = getMenuId(baseId);

  if (ariaControls) {
    return ariaControls + " " + menuId;
  }

  return menuId;
}

function getAutocomplete(options) {
  if (options.list && options.inline) return "both";
  if (options.list) return "list";
  if (options.inline) return "inline";
  return "none";
}

function isFirstItemAutoSelected(options) {
  var firstItem = options.items.find(function (item) {
    return !item.disabled;
  });
  return options.autoSelect && options.currentId && (firstItem === null || firstItem === void 0 ? void 0 : firstItem.id) === options.currentId;
}

function hasCompletionString(inputValue, currentValue) {
  return !!currentValue && currentValue.length > inputValue.length && currentValue.toLowerCase().indexOf(inputValue.toLowerCase()) === 0;
}

function getCompletionString(inputValue, currentValue) {
  if (!currentValue) return "";
  var index = currentValue.toLowerCase().indexOf(inputValue.toLowerCase());
  return currentValue.slice(index + inputValue.length);
}

function getValue(options) {
  if (!options.inline) {
    return options.inputValue;
  }

  if (options.autoSelect && isFirstItemAutoSelected(options)) {
    if (hasCompletionString(options.inputValue, options.currentValue)) {
      return options.inputValue + getCompletionString(options.inputValue, options.currentValue);
    }

    return options.inputValue;
  }

  return options.currentValue || options.inputValue;
}

function getFirstEnabledItemId(items) {
  var _items$find;

  return (_items$find = items.find(function (item) {
    return !item.disabled;
  })) === null || _items$find === void 0 ? void 0 : _items$find.id;
}

function useCompletionString(ref, _ref) {
  var items = _ref.items,
      inputValue = _ref.inputValue,
      currentValue = _ref.currentValue,
      autoSelect = _ref.autoSelect,
      currentId = _ref.currentId;
  useEffect(function () {
    if (!autoSelect) return;
    if (!currentValue) return;
    if (!hasCompletionString(inputValue, currentValue)) return;
    if (currentId !== getFirstEnabledItemId(items)) return;
    var element = ref.current;

    if (!element) {
      process.env.NODE_ENV !== "production" ? warning(true, "Can't auto select combobox because `ref` wasn't passed to the component", "See https://reakit.io/docs/combobox") : void 0;
      return;
    }

    element.setSelectionRange(inputValue.length, currentValue.length);
  }, [autoSelect, currentValue, inputValue, currentId, items]);
}

var unstable_useCombobox = createHook({
  name: "Combobox",
  compose: useComposite,
  keys: COMBOBOX_KEYS,
  useOptions: function useOptions(_ref2) {
    var _ref2$menuRole = _ref2.menuRole,
        menuRole = _ref2$menuRole === void 0 ? "listbox" : _ref2$menuRole,
        _ref2$hideOnEsc = _ref2.hideOnEsc,
        hideOnEsc = _ref2$hideOnEsc === void 0 ? true : _ref2$hideOnEsc,
        options = _objectWithoutPropertiesLoose(_ref2, ["menuRole", "hideOnEsc"]);

    return _objectSpread2({
      menuRole: menuRole,
      hideOnEsc: hideOnEsc
    }, options);
  },
  useProps: function useProps(options, _ref3) {
    var htmlRef = _ref3.ref,
        htmlOnClick = _ref3.onClick,
        htmlOnChange = _ref3.onChange,
        htmlOnBlur = _ref3.onBlur,
        htmlOnKeyDown = _ref3.onKeyDown,
        ariaControls = _ref3["aria-controls"],
        htmlProps = _objectWithoutPropertiesLoose(_ref3, ["ref", "onClick", "onChange", "onBlur", "onKeyDown", "aria-controls"]);

    var ref = useRef(null);
    var onClickRef = useLiveRef(htmlOnClick);
    var onChangeRef = useLiveRef(htmlOnChange);
    var onBlurRef = useLiveRef(htmlOnBlur);
    var onKeyDownRef = useLiveRef(htmlOnKeyDown);
    var value = getValue(options);
    useCompletionString(ref, options);
    var onClick = useCallback(function (event) {
      var _onClickRef$current, _options$show;

      (_onClickRef$current = onClickRef.current) === null || _onClickRef$current === void 0 ? void 0 : _onClickRef$current.call(onClickRef, event);
      if (event.defaultPrevented) return;
      (_options$show = options.show) === null || _options$show === void 0 ? void 0 : _options$show.call(options);
    }, [options.show]);
    var onChange = useCallback(function (event) {
      var _onChangeRef$current, _options$show2, _options$setCurrentId, _options$setInputValu;

      (_onChangeRef$current = onChangeRef.current) === null || _onChangeRef$current === void 0 ? void 0 : _onChangeRef$current.call(onChangeRef, event);
      if (event.defaultPrevented) return;
      (_options$show2 = options.show) === null || _options$show2 === void 0 ? void 0 : _options$show2.call(options);
      (_options$setCurrentId = options.setCurrentId) === null || _options$setCurrentId === void 0 ? void 0 : _options$setCurrentId.call(options, null);
      (_options$setInputValu = options.setInputValue) === null || _options$setInputValu === void 0 ? void 0 : _options$setInputValu.call(options, event.target.value);
    }, [options.show, options.setCurrentId, options.setInputValue]);
    var onBlur = useCallback(function (event) {
      var _onBlurRef$current;

      (_onBlurRef$current = onBlurRef.current) === null || _onBlurRef$current === void 0 ? void 0 : _onBlurRef$current.call(onBlurRef, event);
      if (event.defaultPrevented) return;
      options.setInputValue(value);
    }, [options.setInputValue, value]);
    var onKeyDown = useCallback(function (event) {
      var _onKeyDownRef$current;

      (_onKeyDownRef$current = onKeyDownRef.current) === null || _onKeyDownRef$current === void 0 ? void 0 : _onKeyDownRef$current.call(onKeyDownRef, event);
      if (event.defaultPrevented) return;

      if (event.key === "Escape" && options.hideOnEsc) {
        var _options$hide;

        (_options$hide = options.hide) === null || _options$hide === void 0 ? void 0 : _options$hide.call(options);
      }
    }, [options.hideOnEsc, options.hide]);
    return _objectSpread2({
      ref: useForkRef(ref, useForkRef(options.unstable_referenceRef, htmlRef)),
      role: "combobox",
      autoComplete: "off",
      "aria-controls": getControls(options.baseId, ariaControls),
      "aria-haspopup": options.menuRole,
      "aria-expanded": options.visible,
      "aria-autocomplete": getAutocomplete(options),
      value: value,
      onClick: onClick,
      onChange: onChange,
      onBlur: onBlur,
      onKeyDown: onKeyDown
    }, htmlProps);
  },
  useComposeProps: function useComposeProps(options, _ref4) {
    var onKeyUp = _ref4.onKeyUp,
        htmlOnKeyDown = _ref4.onKeyDown,
        htmlProps = _objectWithoutPropertiesLoose(_ref4, ["onKeyUp", "onKeyDown"]);

    var compositeHTMLProps = useComposite(options, htmlProps, true);
    var onKeyDown = useCallback(function (event) {
      var _compositeHTMLProps$o;

      var inputHasFocus = options.currentId === null;
      htmlOnKeyDown === null || htmlOnKeyDown === void 0 ? void 0 : htmlOnKeyDown(event);

      if (inputHasFocus) {
        if (event.key === "ArrowLeft" || event.key === "ArrowRight" || event.key === "Home" || event.key === "End") {
          // Do not perform list actions when pressing horizontal arrow keys
          // when focusing the combobox input while no option has focus.
          return;
        }

        if (!event.defaultPrevented && !event.ctrlKey && !event.altKey && !event.shiftKey && !event.metaKey && (event.key === "ArrowUp" || event.key === "ArrowDown" || event.key.length === 1)) {
          var _options$show3;

          // Up/Down arrow keys and printable characters should open the
          // combobox popover.
          (_options$show3 = options.show) === null || _options$show3 === void 0 ? void 0 : _options$show3.call(options);
        }
      } else if (options.menuRole === "grid") {
        // If menu is a grid and there's an option with focus, we don't want
        // navigation keys to move the caret on the combobox input.
        if (event.key === "ArrowUp" || event.key === "ArrowDown" || event.key === "ArrowLeft" || event.key === "ArrowRight" || event.key === "Home" || event.key === "End" || event.key === "PageUp" || event.key === "PageDown") {
          event.preventDefault();
        }
      } else if (event.key === "ArrowUp" || event.key === "ArrowDown") {
        // If menu is a one-dimensional list and there's an option with
        // focus, we don't want Up/Down arrow keys to move the caret on the
        // combobox input.
        event.preventDefault();
      } else if (event.key === "Home" || event.key === "End" || event.key.length === 1) {
        // If menu is a one-dimensional list and there's an option with
        // focus, we don't want Home/End and printable characters to perform
        // actions on the option, only on the combobox input.
        return;
      }

      (_compositeHTMLProps$o = compositeHTMLProps.onKeyDown) === null || _compositeHTMLProps$o === void 0 ? void 0 : _compositeHTMLProps$o.call(compositeHTMLProps, event);
    }, [htmlOnKeyDown, options.show, options.menuRole, compositeHTMLProps.onKeyDown]);
    return _objectSpread2(_objectSpread2({}, compositeHTMLProps), {}, {
      onKeyDown: onKeyDown,
      onKeyUp: onKeyUp
    });
  }
});
var unstable_Combobox = createComponent({
  as: "input",
  memo: true,
  useHook: unstable_useCombobox
});

export { unstable_Combobox, unstable_useCombobox };
